﻿using System;
using System.Linq;
using System.Collections.Generic;


public static class EnumerableExtensions
{
    /// <summary>
    /// Creates a HashSet of TSource from an IEnumerable of TSource using the identity selector and default equality comparer.
    /// </summary>
    /// <typeparam name="TSource">The type of elements of source.</typeparam>
    /// <param name="source">An IEnumerable of TSource to create a HashSet of TSource from.</param>
    /// <returns>A HashSet of TSource that contains the unique elements.</returns>
    public static HashSet<TSource> ToHashSet<TSource>(this IEnumerable<TSource> source)
    {
        return source.ToHashSet<TSource, TSource>(item => item, null);
    }

    /// <summary>
    /// Creates a HashSet of TSource from an IEnumerable of TSource using the identity selector and the specified
    /// equality comparer for TSource.
    /// </summary>
    /// <typeparam name="TSource">The type of elements of source.</typeparam>
    /// <param name="source">An IEnumerable of TSource to create a HashSet of TSource from.</param>
    /// <param name="comparer">A comparer that compares two elements of type TSource.</param>
    /// <returns>A HashSet of TSource that contains the unique elements.</returns>
    public static HashSet<TSource> ToHashSet<TSource>(this IEnumerable<TSource> source, IEqualityComparer<TSource> comparer)
    {
        return source.ToHashSet<TSource, TSource>(item => item, comparer);
    }

    /// <summary>
    /// Creates a HashSet of TElement from an IEnumerable of TSource using the specified element selector and default equality comparer.
    /// </summary>
    /// <typeparam name="TSource">The type of elements of source.</typeparam>
    /// <typeparam name="TElement">The type of elements returned by the element selector.</typeparam>
    /// <param name="source">An IEnumerable of TSource to create a HashSet of TElement from.</param>
    /// <param name="elementSelector">A function to extract an element value from each source element.</param>
    /// <returns>A HashSet of TElement that contains the unique elements.</returns>
    public static HashSet<TElement> ToHashSet<TSource, TElement>(this IEnumerable<TSource> source, Func<TSource, TElement> elementSelector)
    {
        return source.ToHashSet<TSource, TElement>(elementSelector, null);
    }

    /// <summary>
    /// Creates a HashSet of TElement from an IEnumerable of TSource using the specified element selector and the specified
    /// equality comparer for TElement.
    /// </summary>
    /// <typeparam name="TSource">The type of elements of source.</typeparam>
    /// <typeparam name="TElement">The type of elements returned by the element selector.</typeparam>
    /// <param name="source">An IEnumerable of TSource to create a HashSet of TElement from.</param>
    /// <param name="elementSelector">A function to extract an element value from each source element.</param>
    /// <param name="comparer">A comparer that compares two elements of type TElement.</param>
    /// <returns>A HashSet of TElement that contains the unique elements.</returns>
    public static HashSet<TElement> ToHashSet<TSource, TElement>(this IEnumerable<TSource> source, Func<TSource, TElement> elementSelector, IEqualityComparer<TElement> comparer)
    {
        if (source == null) throw new ArgumentNullException("source");
        if (elementSelector == null) throw new ArgumentNullException("elementSelector");

        // you can unroll this into a foreach if you want efficiency gain, but for brevity...
        return new HashSet<TElement>(source.Select(elementSelector), comparer);
    }
}
